# coding: utf-8

translations = {
    "volume": {
        "pt": "Volume",
        "es": "Volumen",
        "en": "Volume"
    },
    "number": {
        "pt": u"Número",
        "es": "Numero",
        "en": "Number"
    },
    "pages": {
        "pt": u"Páginas",
        "es": "Paginas",
        "en": "Pages"
    },
    "supplement": {
        "pt": "Suplemento",
        "es": "Suplemento",
        "en": "Supplement"
    },
    "suppl": {
        "pt": "supl",
        "es": "supl",
        "en": "suppl"
    },
    "article number": {
        "pt": "Número do artigo",
        "es": "Numero del articulo",
        "en": "Article number"
    },
}
